@@ -9,7 +9,7 @@ use solana_sdk::{
     account_utils::StateMut,
     bpf_loader_upgradeable::{self, UpgradeableLoaderState},
     feature_set::{
-        cpi_share_ro_and_exec_accounts, demote_sysvar_write_locks, instructions_sysvar_enabled,
+        cpi_share_ro_and_exec_accounts, instructions_sysvar_enabled, updated_verify_policy,
         FeatureSet,
     },
     ic_msg,
@@ -105,6 +105,7 @@ impl PreAccount {
         rent: &Rent,
         post: &AccountSharedData,
         timings: &mut ExecuteDetailsTimings,
+        updated_verify_policy: bool,
     ) -> Result<(), InstructionError> {
         let pre = self.account.borrow();
 
@@ -173,9 +174,14 @@ impl PreAccount {
             if !rent.is_exempt(post.lamports, post.data().len()) {
                 return Err(InstructionError::ExecutableAccountNotRentExempt);
             }
+            let owner = if updated_verify_policy {
+                post.owner()
+            } else {
+                pre.owner()
+            };
             if !is_writable // line coverage used to get branch coverage
                 || pre.executable
-                || *program_id != pre.owner
+                || program_id != owner
             {
                 return Err(InstructionError::ExecutableModified);
             }
@@ -353,7 +359,7 @@ impl<'a> InvokeContext for ThisInvokeContext<'a> {
                 &self.rent,
                 caller_write_privileges,
                 &mut self.timings,
-                self.feature_set.is_active(&demote_sysvar_write_locks::id()),
+                self.feature_set.is_active(&updated_verify_policy::id()),
             ),
             None => Err(InstructionError::GenericError), // Should never happen
         }
@@ -566,7 +572,6 @@ impl MessageProcessor {
         instruction: &'a CompiledInstruction,
         executable_accounts: &'a [(Pubkey, Rc<RefCell<AccountSharedData>>)],
         accounts: &'a [Rc<RefCell<AccountSharedData>>],
-        demote_sysvar_write_locks: bool,
     ) -> Vec<KeyedAccount<'a>> {
         let mut keyed_accounts = create_keyed_readonly_accounts(&executable_accounts);
         let mut keyed_accounts2: Vec<_> = instruction
@@ -577,7 +582,7 @@ impl MessageProcessor {
                 let index = index as usize;
                 let key = &message.account_keys[index];
                 let account = &accounts[index];
-                if message.is_writable(index, demote_sysvar_write_locks) {
+                if message.is_writable(index) {
                     KeyedAccount::new(key, is_signer, account)
                 } else {
                     KeyedAccount::new_readonly(key, is_signer, account)
@@ -722,14 +727,7 @@ impl MessageProcessor {
     ) -> Result<(), InstructionError> {
         let invoke_context = RefCell::new(invoke_context);
 
-        let (
-            message,
-            executables,
-            accounts,
-            account_refs,
-            caller_write_privileges,
-            demote_sysvar_write_locks,
-        ) = {
+        let (message, executables, accounts, account_refs, caller_write_privileges) = {
             let invoke_context = invoke_context.borrow();
 
             let caller_program_id = invoke_context.get_caller()?;
@@ -821,7 +819,6 @@ impl MessageProcessor {
                 accounts,
                 account_refs,
                 caller_write_privileges,
-                invoke_context.is_feature_active(&demote_sysvar_write_locks::id()),
             )
         };
 
@@ -840,7 +837,7 @@ impl MessageProcessor {
             let invoke_context = invoke_context.borrow();
             for (i, (account, account_ref)) in accounts.iter().zip(account_refs).enumerate() {
                 let account = account.borrow();
-                if message.is_writable(i, demote_sysvar_write_locks) && !account.executable {
+                if message.is_writable(i) && !account.executable {
                     account_ref.try_account_ref_mut()?.lamports = account.lamports;
                     account_ref.try_account_ref_mut()?.owner = account.owner;
                     if account_ref.data_len()? != account.data().len()
@@ -883,16 +880,9 @@ impl MessageProcessor {
                 accounts,
                 Some(caller_write_privileges),
             )?;
-            let demote_sysvar_write_locks =
-                invoke_context.is_feature_active(&demote_sysvar_write_locks::id());
             // Construct keyed accounts
-            let keyed_accounts = Self::create_keyed_accounts(
-                message,
-                instruction,
-                executable_accounts,
-                accounts,
-                demote_sysvar_write_locks,
-            );
+            let keyed_accounts =
+                Self::create_keyed_accounts(message, instruction, executable_accounts, accounts);
 
             // Invoke callee
             invoke_context.push(program_id)?;
@@ -954,6 +944,7 @@ impl MessageProcessor {
     }
 
     /// Verify the results of an instruction
+    #[allow(clippy::too_many_arguments)]
     pub fn verify(
         message: &Message,
         instruction: &CompiledInstruction,
@@ -962,7 +953,7 @@ impl MessageProcessor {
         accounts: &[Rc<RefCell<AccountSharedData>>],
         rent: &Rent,
         timings: &mut ExecuteDetailsTimings,
-        demote_sysvar_write_locks: bool,
+        updated_verify_policy: bool,
     ) -> Result<(), InstructionError> {
         // Verify all executable accounts have zero outstanding refs
         Self::verify_account_references(executable_accounts)?;
@@ -981,10 +972,11 @@ impl MessageProcessor {
                 let account = accounts[account_index].borrow();
                 pre_accounts[unique_index].verify(
                     &program_id,
-                    message.is_writable(account_index, demote_sysvar_write_locks),
+                    message.is_writable(account_index),
                     rent,
                     &account,
                     timings,
+                    updated_verify_policy,
                 )?;
                 pre_sum += u128::from(pre_accounts[unique_index].lamports());
                 post_sum += u128::from(account.lamports);
@@ -1001,6 +993,7 @@ impl MessageProcessor {
     }
 
     /// Verify the results of a cross-program instruction
+    #[allow(clippy::too_many_arguments)]
     fn verify_and_update(
         message: &Message,
         instruction: &CompiledInstruction,
@@ -1010,7 +1003,7 @@ impl MessageProcessor {
         rent: &Rent,
         caller_write_privileges: Option<&[bool]>,
         timings: &mut ExecuteDetailsTimings,
-        demote_sysvar_write_locks: bool,
+        updated_verify_policy: bool,
     ) -> Result<(), InstructionError> {
         // Verify the per-account instruction results
         let (mut pre_sum, mut post_sum) = (0_u128, 0_u128);
@@ -1021,7 +1014,7 @@ impl MessageProcessor {
                 let is_writable = if let Some(caller_write_privileges) = caller_write_privileges {
                     caller_write_privileges[account_index]
                 } else {
-                    message.is_writable(account_index, demote_sysvar_write_locks)
+                    message.is_writable(account_index)
                 };
                 // Find the matching PreAccount
                 for pre_account in pre_accounts.iter_mut() {
@@ -1033,7 +1026,14 @@ impl MessageProcessor {
                                 .map_err(|_| InstructionError::AccountBorrowOutstanding)?;
                         }
                         let account = account.borrow();
-                        pre_account.verify(&program_id, is_writable, &rent, &account, timings)?;
+                        pre_account.verify(
+                            &program_id,
+                            is_writable,
+                            &rent,
+                            &account,
+                            timings,
+                            updated_verify_policy,
+                        )?;
                         pre_sum += u128::from(pre_account.lamports());
                         post_sum += u128::from(account.lamports);
                         if is_writable && !pre_account.executable() {
@@ -1075,7 +1075,6 @@ impl MessageProcessor {
         feature_set: Arc<FeatureSet>,
         bpf_compute_budget: BpfComputeBudget,
         timings: &mut ExecuteDetailsTimings,
-        demote_sysvar_write_locks: bool,
         account_db: Arc<Accounts>,
         ancestors: &Ancestors,
     ) -> Result<(), InstructionError> {
@@ -1111,13 +1110,8 @@ impl MessageProcessor {
             account_db,
             ancestors,
         );
-        let keyed_accounts = Self::create_keyed_accounts(
-            message,
-            instruction,
-            executable_accounts,
-            accounts,
-            demote_sysvar_write_locks,
-        );
+        let keyed_accounts =
+            Self::create_keyed_accounts(message, instruction, executable_accounts, accounts);
         self.process_instruction(
             program_id,
             &keyed_accounts,
@@ -1132,7 +1126,7 @@ impl MessageProcessor {
             accounts,
             &rent_collector.rent,
             timings,
-            demote_sysvar_write_locks,
+            invoke_context.is_feature_active(&updated_verify_policy::id()),
         )?;
 
         timings.accumulate(&invoke_context.timings);
@@ -1161,7 +1155,6 @@ impl MessageProcessor {
         account_db: Arc<Accounts>,
         ancestors: &Ancestors,
     ) -> Result<(), TransactionError> {
-        let demote_sysvar_write_locks = feature_set.is_active(&demote_sysvar_write_locks::id());
         for (instruction_index, instruction) in message.instructions.iter().enumerate() {
             let instruction_recorder = instruction_recorders
                 .as_ref()
@@ -1180,7 +1173,6 @@ impl MessageProcessor {
                 feature_set.clone(),
                 bpf_compute_budget,
                 timings,
-                demote_sysvar_write_locks,
                 account_db.clone(),
                 ancestors,
             )
@@ -1414,6 +1406,7 @@ mod tests {
                 &self.rent,
                 &self.post,
                 &mut ExecuteDetailsTimings::default(),
+                true,
             )
         }
     }
@@ -1736,6 +1729,21 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_verify_account_changes_owner_executable() {
+        let alice_program_id = solana_sdk::pubkey::new_rand();
+        let bob_program_id = solana_sdk::pubkey::new_rand();
+
+        assert_eq!(
+            Change::new(&alice_program_id, &alice_program_id)
+                .owner(&bob_program_id)
+                .executable(false, true)
+                .verify(),
+            Err(InstructionError::ExecutableModified),
+            "Program should not be able to change owner and executable at the same time"
+        );
+    }
+
     #[test]
     fn test_process_message_readonly_handling() {
         #[derive(Serialize, Deserialize)]
@@ -2133,6 +2141,8 @@ mod tests {
         ];
         let programs: Vec<(_, ProcessInstructionWithContext)> =
             vec![(callee_program_id, mock_process_instruction)];
+        let feature_set = FeatureSet::all_enabled();
+
         let ancestors = Ancestors::default();
         let mut invoke_context = ThisInvokeContext::new(
             &caller_program_id,
@@ -2149,7 +2159,7 @@ mod tests {
             BpfComputeBudget::default(),
             Rc::new(RefCell::new(Executors::default())),
             None,
-            Arc::new(FeatureSet::all_enabled()),
+            Arc::new(feature_set),
             Arc::new(Accounts::default()),
             &ancestors,
         );
@@ -2165,13 +2175,12 @@ mod tests {
             &MockInstruction::NoopSuccess,
             metas.clone(),
         );
-        let demote_sysvar_write_locks = true;
         let message = Message::new(&[instruction], None);
         let caller_write_privileges = message
             .account_keys
             .iter()
             .enumerate()
-            .map(|(i, _)| message.is_writable(i, demote_sysvar_write_locks))
+            .map(|(i, _)| message.is_writable(i))
             .collect::<Vec<bool>>();
         assert_eq!(
             MessageProcessor::process_cross_program_instruction(
@@ -2206,7 +2215,7 @@ mod tests {
                 .account_keys
                 .iter()
                 .enumerate()
-                .map(|(i, _)| message.is_writable(i, demote_sysvar_write_locks))
+                .map(|(i, _)| message.is_writable(i))
                 .collect::<Vec<bool>>();
             assert_eq!(
                 MessageProcessor::process_cross_program_instruction(
